#include <LiquidCrystal.h>

#define VR1 4700
#define VR2 15000
#define RR2 1000

#define RR3 50000

#define VOLTMETER A0
#define OHMMETER A1
#define OHMMETER2 A2
#define BUTTON 2
#define BACKLIGHT 6

#define MODE_VOLT 0
#define MODE_RES_1K 1
#define MODE_RES_50K 2

int mode = MODE_VOLT;
LiquidCrystal lcd(8,9,10,11,12,13); //Creates the lcd object

double calculateVoltage()
{
  double vout, vin, ratio;
  vout = 5 * analogRead(VOLTMETER) / (double)1024;
  ratio = VR1 / (double)(VR1+VR2);
  vin = vout / ratio;
  
  return vin;
}

long calculateResistance()
{
  double vout, numerator, denominator;
  vout = 5 * analogRead(OHMMETER) / (double)1024;
  numerator = vout * RR2;
  denominator = 5 - vout;
  if(analogRead(OHMMETER) > 1000)
    return -1;
  else
    return numerator / denominator;  
}

long calculateHighResistance()
{
  double vout, numerator, denominator;
  vout = 5 * analogRead(OHMMETER2) / (double)1024;
  numerator = vout * RR3;
  denominator = 5 - vout;
  if(analogRead(OHMMETER2) > 1000)
    return -1;
  else
    return numerator / denominator;  
}

void switchMode()
{
  mode++;
  mode %= 3;
}

void setup() {
  pinMode(VOLTMETER, INPUT);
  pinMode(OHMMETER, INPUT);
  pinMode(OHMMETER2, INPUT);
  
  lcd.begin(20,4); //Sets up the LCD as (20 characters) x (4 lines)
  attachInterrupt(digitalPinToInterrupt(BUTTON), switchMode, RISING); 
  
}

void loop() {
  analogWrite(BACKLIGHT, 100);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Mode:");
  lcd.setCursor(0,2);
  lcd.print("Value:");
  if(mode == MODE_VOLT)
  {
    lcd.setCursor(6,0);
    lcd.print("Voltage");
    lcd.setCursor(7,2);
    lcd.print(calculateVoltage());
  }
  else if(mode == MODE_RES_1K)
  {
    lcd.setCursor(6,0);
    lcd.print("Resistance");
    lcd.setCursor(7,2);
    int resistance = calculateResistance();
    if(resistance == -1)
      lcd.print("OL");
    else
      lcd.print(calculateResistance());
  }
  else if(mode == MODE_RES_50K)
  {
    lcd.setCursor(6,0);
    lcd.print("Resistance (high)");
    lcd.setCursor(7,2);
    int resistance = calculateHighResistance();
    if(resistance == -1)
      lcd.print("OL");
    else
      lcd.print(calculateHighResistance());
  }
  calculateResistance();
  delay(250);
}






